/*
 * EndSymbol.h
 *
 *  Created on: Jun 19, 2014
 *      Author: Jan Travnicek
 */

#ifndef END_SYMBOL_H_
#define END_SYMBOL_H_

#include "SymbolBase.h"
#include "Symbol.h"
#include <object/Object.h>

namespace alphabet {

/**
 * Represents blank symbol in an alphabet.
 */
class EndSymbol : public SymbolBase {
public:
	/**
	 * Creates a blank symbol.
	 * @param symbol name of the symbol
	 */
	explicit EndSymbol ( );

	virtual SymbolBase * clone ( ) const;
	virtual SymbolBase * plunder ( ) &&;

	virtual int compare ( const ObjectBase & other ) const {
		if ( ext::type_index ( typeid ( * this ) ) == ext::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return ext::type_index ( typeid ( * this ) ) - ext::type_index ( typeid ( other ) );
	}

	virtual int compare ( const EndSymbol & other ) const;

	virtual void operator >>( std::ostream & out ) const;

	virtual explicit operator std::string ( ) const;

	template < typename Base >
	static inline typename std::enable_if <   std::is_integral < Base >::value, Base >::type instance ( );
	template < typename Base >
	static inline typename std::enable_if < ! std::is_integral < Base >::value, Base >::type instance ( );

	static const std::string & getXmlTagName() {
		static std::string xmlTagName = "EndSymbol";

		return xmlTagName;
	}

	static EndSymbol parse ( ext::deque < sax::Token >::iterator & input );

	void compose ( ext::deque < sax::Token > & out ) const;

	virtual SymbolBase * inc ( ) &&;

	typedef EndSymbol normalized_type;

	virtual SymbolBase * normalize ( ) && {
		return this;
	}
};

template < typename Base >
inline typename std::enable_if < std::is_integral < Base >::value, Base >::type EndSymbol::instance ( ) {
	return '$';
}

template < >
inline std::string EndSymbol::instance < std::string > ( ) {
	return std::string ( 1, EndSymbol::instance < char > ( ) );
}

template < >
inline EndSymbol EndSymbol::instance < EndSymbol > ( ) {
	return EndSymbol ( );
}

template < >
inline Symbol EndSymbol::instance < Symbol > ( ) {
	return Symbol ( EndSymbol ( ) );
}

template < >
inline alib::Object EndSymbol::instance < alib::Object > ( ) {
	return alib::Object ( EndSymbol ( ) );
}

} /* namespace alphabet */

#endif /* END_SYMBOL_H_ */
