/*
 * InitialSymbol.h
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#ifndef INITIAL_SYMBOL_LABEL_H_
#define INITIAL_SYMBOL_LABEL_H_

#include "SymbolBase.h"
#include "Symbol.h"
#include <object/Object.h>

namespace alphabet {

/**
 * Represents symbol in an alphabet.
 */
class InitialSymbol : public SymbolBase {
public:
	/**
	 * Creates new symbol with given name.
	 * @param symbol name of the symbol
	 */
	explicit InitialSymbol ( );

	virtual SymbolBase * clone ( ) const;

	virtual SymbolBase * plunder ( ) &&;

	virtual int compare ( const ObjectBase & other ) const {
		if ( ext::type_index ( typeid ( * this ) ) == ext::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return ext::type_index ( typeid ( * this ) ) - ext::type_index ( typeid ( other ) );
	}

	virtual int compare ( const InitialSymbol & other ) const;

	virtual void operator >>( std::ostream & ) const;

	virtual explicit operator std::string ( ) const;

	template < typename Base >
	static inline typename std::enable_if <   std::is_integral < Base >::value, Base >::type instance ( );
	template < typename Base >
	static inline typename std::enable_if < ! std::is_integral < Base >::value, Base >::type instance ( );

	static const std::string & getXmlTagName() {
		static std::string xmlTagName = "InitialSymbol";

		return xmlTagName;
	}

	static InitialSymbol parse ( ext::deque < sax::Token >::iterator & input );

	void compose ( ext::deque < sax::Token > & out ) const;

	virtual SymbolBase * inc ( ) &&;

	typedef InitialSymbol normalized_type;

	virtual SymbolBase * normalize ( ) && {
		return this;
	}
};

template < typename Base >
inline typename std::enable_if < std::is_integral < Base >::value, Base >::type InitialSymbol::instance ( ) {
	return 0;
}

template < >
inline std::string InitialSymbol::instance < std::string > ( ) {
	return std::string ( "S" );
}

template < >
inline InitialSymbol InitialSymbol::instance < InitialSymbol > ( ) {
	return InitialSymbol ( );
}

template < >
inline Symbol InitialSymbol::instance < Symbol > ( ) {
	return Symbol ( InitialSymbol ( ) );
}

template < >
inline alib::Object InitialSymbol::instance < alib::Object > ( ) {
	return alib::Object ( InitialSymbol ( ) );
}

} /* namespace alphabet */

#endif /* INITIAL_SYMBOL_LABEL_H_ */
