/*
 * UniqueSymbol.cpp
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travicek
 */

#include "UniqueSymbol.h"
#include <sstream>
#include <sax/FromXMLParserHelper.h>
#include "Symbol.h"
#include <object/Object.h>
#include <registration/TypeRegistration.hpp>

namespace alphabet {

UniqueSymbol::UniqueSymbol(Symbol symbol, primitive::Integer id) : m_symbol(std::move(symbol)), m_id(std::move(id)) {

}

SymbolBase* UniqueSymbol::clone() const {
	return new UniqueSymbol(*this);
}

SymbolBase* UniqueSymbol::plunder() && {
	return new UniqueSymbol(std::move(*this));
}

const Symbol& UniqueSymbol::getSymbol() const {
	return m_symbol;
}

const primitive::Integer& UniqueSymbol::getId() const {
	return m_id;
}

primitive::Integer& UniqueSymbol::getId() {
	return m_id;
}

int UniqueSymbol::compare(const UniqueSymbol& other) const {
	int res = m_symbol.getData().compare(other.m_symbol.getData());
	if(res == 0) res = m_id.compare(other.m_id);
	return res;
}

void UniqueSymbol::operator>>(std::ostream& out) const {
	out << "(UniqueSymbol " << m_symbol << ", " << m_id << ")";
}

UniqueSymbol::operator std::string() const {
	std::stringstream ss;
	ss << "{";
	ss << m_symbol;
	ss << ", ";
	ss << m_id;
	ss << "}";
	return std::move(ss).str();
}

UniqueSymbol UniqueSymbol::parse(ext::deque<sax::Token>::iterator& input) {
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, UniqueSymbol::getXmlTagName());

	Symbol firstSymbol = alib::xmlApi<Symbol>::parse(input);
	primitive::Integer secondSymbol = alib::xmlApi<primitive::Integer>::parse(input);

	UniqueSymbol data(std::move(firstSymbol), std::move(secondSymbol));
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, UniqueSymbol::getXmlTagName());
	return data;
}

void UniqueSymbol::compose(ext::deque<sax::Token>& out) const {
	out.emplace_back(UniqueSymbol::getXmlTagName(), sax::Token::TokenType::START_ELEMENT);
	alib::xmlApi<alphabet::Symbol>::compose(out, m_symbol);
	alib::xmlApi<primitive::Integer>::compose(out, m_id);
	out.emplace_back(UniqueSymbol::getXmlTagName(), sax::Token::TokenType::END_ELEMENT);
}

SymbolBase* UniqueSymbol::inc() && {
	std::move(m_id).inc();
	return NULL;
}

} /* namespace alphabet */

namespace alib {

auto uniqueSymbolType = registration::TypeRegister < alphabet::Symbol, alphabet::UniqueSymbol > ();
auto uniqueSymbolType2 = registration::TypeRegister < alib::Object, alphabet::UniqueSymbol > ();

} /* namespace alib */
