/*
 * FinalStateLabel.h
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#ifndef FINAL_STATE_LABEL_H_
#define FINAL_STATE_LABEL_H_

#include "LabelBase.h"
#include "Label.h"
#include <object/Object.h>

namespace label {

/**
 * Represents symbol in an alphabet.
 */
class FinalStateLabel final : public LabelBase {
public:
	/**
	 * Creates new symbol with given name.
	 * @param symbol name of the symbol
	 */
	explicit FinalStateLabel ( );

	virtual LabelBase * clone ( ) const;

	virtual LabelBase * plunder ( ) &&;

	virtual int compare ( const ObjectBase & other ) const {
		if ( ext::type_index ( typeid ( * this ) ) == ext::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return ext::type_index ( typeid ( * this ) ) - ext::type_index ( typeid ( other ) );
	}

	virtual int compare ( const FinalStateLabel & other ) const;

	virtual void operator >>( std::ostream & ) const;

	virtual explicit operator std::string ( ) const;

	template < typename Base >
	static inline typename std::enable_if <   std::is_integral < Base >::value, Base >::type instance ( );
	template < typename Base >
	static inline typename std::enable_if < ! std::is_integral < Base >::value, Base >::type instance ( );

	static const std::string & getXmlTagName() {
		static std::string xmlTagName = "FinalStateLabel";

		return xmlTagName;
	}

	static FinalStateLabel parse ( ext::deque < sax::Token >::iterator & input );

	void compose ( ext::deque < sax::Token > & out ) const;

	virtual LabelBase * inc ( ) &&;

	typedef FinalStateLabel normalized_type;

	virtual LabelBase * normalize ( ) && {
		return this;
	}
};

template < typename Base >
inline typename std::enable_if < std::is_integral < Base >::value, Base >::type FinalStateLabel::instance ( ) {
	return std::numeric_limits < Base >::max();
}

template < >
inline std::string FinalStateLabel::instance < std::string > ( ) {
	return std::string ( "final" );
}

template < >
inline FinalStateLabel FinalStateLabel::instance < FinalStateLabel > ( ) {
	return FinalStateLabel ( );
}

template < >
inline Label FinalStateLabel::instance < Label > ( ) {
	return Label ( FinalStateLabel ( ) );
}

template < >
inline alib::Object FinalStateLabel::instance < alib::Object > ( ) {
	return alib::Object ( FinalStateLabel ( ) );
}

} /* namespace label */

#endif /* FINAL_STATE_LABEL_H_ */
