/*
 * FormalRTEEmpty.h
 *
 *  Created on: Feb 20, 2016
 *      Author: Tomas Pecka
 */

#ifndef FORMAL_RTE_EMPTY_H_
#define FORMAL_RTE_EMPTY_H_

#include "FormalRTEElement.h"
#include <sstream>

namespace rte {

/**
 * Represents empty regular expression in the regular expression.
 */
template < class SymbolType, class RankType >
class FormalRTEEmpty : public FormalRTEElement < SymbolType, RankType > {
public:
	virtual void accept ( typename FormalRTEElement < SymbolType, RankType >::Visitor & visitor ) const override {
		visitor.visit ( * this );
	}

	explicit FormalRTEEmpty ( );

	/**
	 * @copydoc FormalRTEElement::clone() const
	 */
	virtual FormalRTEEmpty < SymbolType, RankType > * clone ( ) const override;

	/**
	 * @copydoc FormalRTEElement::plunder() const
	 */
	virtual FormalRTEEmpty < SymbolType, RankType > * plunder ( ) && override;

	/**
	 * @copydoc FormalRTEElement::testSymbol() const
	 */
	virtual bool testSymbol ( const common::ranked_symbol < SymbolType, RankType > & symbol ) const override;

	/**
	 * @copydoc RTEElement::computeMinimalAlphabet()
	 */
	virtual void computeMinimalAlphabet ( ext::set < common::ranked_symbol < SymbolType, RankType > > & alphabetF, ext::set < common::ranked_symbol < SymbolType, RankType > > & alphabetK ) const override;

	/**
	 * @copydoc FormalRTEElement::checkAlphabet()
	 */
	virtual bool checkAlphabet ( const ext::set < common::ranked_symbol < SymbolType, RankType > > & alphabetF, const ext::set < common::ranked_symbol < SymbolType, RankType > > & alphabetK ) const override;

	virtual int compare ( const FormalRTEElement < SymbolType, RankType > & other ) const override {
		if ( ext::type_index ( typeid ( * this ) ) == ext::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return ext::type_index ( typeid ( * this ) ) - ext::type_index ( typeid ( other ) );
	}

	int compare ( const FormalRTEEmpty & ) const;

	/**
	 * @copydoc FormalRTEElement::operator>>() const
	 */
	virtual void operator >>( std::ostream & out ) const override;

	virtual explicit operator std::string ( ) const override;

	virtual ext::smart_ptr < FormalRTEElement < DefaultSymbolType, DefaultRankType > > normalize ( ) && override {
		return ext::smart_ptr < FormalRTEElement < DefaultSymbolType, DefaultRankType > > ( new FormalRTEEmpty < DefaultSymbolType, DefaultRankType > ( ) );
	}
};

template < class SymbolType, class RankType >
FormalRTEEmpty < SymbolType, RankType >::FormalRTEEmpty ( ) {
	// so that default constructor is available
}

template < class SymbolType, class RankType >
FormalRTEEmpty < SymbolType, RankType > * FormalRTEEmpty < SymbolType, RankType >::clone ( ) const {
	return new FormalRTEEmpty ( * this );
}

template < class SymbolType, class RankType >
FormalRTEEmpty < SymbolType, RankType > * FormalRTEEmpty < SymbolType, RankType >::plunder ( ) && {
	return new FormalRTEEmpty ( std::move ( * this ) );
}

template < class SymbolType, class RankType >
int FormalRTEEmpty < SymbolType, RankType >::compare ( const FormalRTEEmpty & ) const {
	return 0;
}

template < class SymbolType, class RankType >
void FormalRTEEmpty < SymbolType, RankType >::operator >>( std::ostream & out ) const {
	out << "(FormalRTEEmpty)";
}

template < class SymbolType, class RankType >
bool FormalRTEEmpty < SymbolType, RankType >::testSymbol ( const common::ranked_symbol < SymbolType, RankType > & ) const {
	return false;
}

template < class SymbolType, class RankType >
void FormalRTEEmpty < SymbolType, RankType >::computeMinimalAlphabet ( ext::set < common::ranked_symbol < SymbolType, RankType > > & /*alphabetF*/, ext::set < common::ranked_symbol < SymbolType, RankType > > & /*alphabetK*/ ) const {
}

template < class SymbolType, class RankType >
bool FormalRTEEmpty < SymbolType, RankType >::checkAlphabet ( const ext::set < common::ranked_symbol < SymbolType, RankType > > & /* alphabetF */, const ext::set < common::ranked_symbol < SymbolType, RankType > > & /* alphabetK */ ) const {
	return true;
}

template < class SymbolType, class RankType >
FormalRTEEmpty < SymbolType, RankType >::operator std::string ( ) const {
	std::stringstream ss;
	ss << * this;
	return ss.str ( );
}

} /* namespace rte */

#endif /* FORMAL_RTE_EMPTY_H_ */
