/*
 * FormalRTEIteration.h
 *
 *  Created on: Feb 20, 2016
 *      Author: Tomas Pecka
 */

#ifndef FORMAL_RTE_ITERATION_H_
#define FORMAL_RTE_ITERATION_H_

#include "FormalRTEElement.h"
#include "FormalRTESymbolSubst.h"

#include <exception/CommonException.h>

#include <sstream>
#include <utility>

namespace rte {

/**
 * Represents iteration operator in the regular expression. Contains one FormalRTEElement
 * as operand.
 */
template < class SymbolType, class RankType >
class FormalRTEIteration : public FormalRTEElement < SymbolType, RankType >, public ext::UnaryNode < ext::smart_ptr < FormalRTEElement < SymbolType, RankType > >, ext::smart_ptr < const FormalRTEElement < SymbolType, RankType > >, FormalRTEIteration < SymbolType, RankType > > {
protected:
	ext::smart_ptr < FormalRTESymbolSubst < SymbolType, RankType > > m_substitutionSymbol;

public:
	virtual void accept ( typename FormalRTEElement < SymbolType, RankType >::Visitor & visitor ) const override {
		visitor.visit ( * this );
	}

	explicit FormalRTEIteration ( FormalRTEElement < SymbolType, RankType > &&, FormalRTESymbolSubst < SymbolType, RankType > );
	explicit FormalRTEIteration ( const FormalRTEElement < SymbolType, RankType > &, FormalRTESymbolSubst < SymbolType, RankType > );

	/**
	 * @copydoc FormalRTEElement::clone() const
	 */
	virtual FormalRTEIteration < SymbolType, RankType > * clone ( ) const override;

	/**
	 * @copydoc FormalRTEElement::plunder() const
	 */
	virtual FormalRTEIteration < SymbolType, RankType > * plunder ( ) && override;

	/**
	 * @copydoc FormalRTEElement::testSymbol() const
	 */
	virtual bool testSymbol ( const common::ranked_symbol < SymbolType, RankType > & symbol ) const override;

	/**
	 * @copydoc FormalRTEElement::computeMinimalAlphabet()
	 */
	virtual void computeMinimalAlphabet ( ext::set < common::ranked_symbol < SymbolType, RankType > > & alphabetF, ext::set < common::ranked_symbol < SymbolType, RankType > > & alphabetK ) const override;

	/**
	 * @copydoc FormalRTEElement::checkAlphabet()
	 */
	virtual bool checkAlphabet ( const ext::set < common::ranked_symbol < SymbolType, RankType > > & alphabetF, const ext::set < common::ranked_symbol < SymbolType, RankType > > & alphabetK ) const override;

	/**
	 * @return element
	 */
	const FormalRTEElement < SymbolType, RankType > & getElement ( ) const;
	const FormalRTESymbolSubst < SymbolType, RankType > & getSubstitutionSymbol ( ) const;

	/**
	 * @return element
	 */
	FormalRTEElement < SymbolType, RankType > & getElement ( );
	FormalRTESymbolSubst < SymbolType, RankType > & getSubstitutionSymbol ( );

	/**
	 * @param element to iterate
	 */
	void setElement ( const FormalRTEElement < SymbolType, RankType > & element );
	void setElement ( FormalRTEElement < SymbolType, RankType > && element );

	void setSubstitutionSymbol ( FormalRTESymbolSubst < SymbolType, RankType > element );

	virtual int compare ( const FormalRTEElement < SymbolType, RankType > & other ) const override {
		if ( ext::type_index ( typeid ( * this ) ) == ext::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return ext::type_index ( typeid ( * this ) ) - ext::type_index ( typeid ( other ) );
	}

	int compare ( const FormalRTEIteration & ) const;

	/**
	 * @copydoc FormalRTEElement::operator>>() const
	 */
	virtual void operator >>( std::ostream & out ) const override;

	virtual explicit operator std::string ( ) const override;

	virtual ext::smart_ptr < FormalRTEElement < DefaultSymbolType, DefaultRankType > > normalize ( ) && override {
		return ext::smart_ptr < FormalRTEElement < DefaultSymbolType, DefaultRankType > > ( new FormalRTEIteration < DefaultSymbolType, DefaultRankType > ( std::move ( * std::move ( getElement ( ) ).normalize ( ) ), std::move ( * m_substitutionSymbol ).normalizeRaw ( ) ) );
	}
};

template < class SymbolType, class RankType >
FormalRTEIteration < SymbolType, RankType >::FormalRTEIteration ( FormalRTEElement < SymbolType, RankType > && element, FormalRTESymbolSubst < SymbolType, RankType > substitutionSymbol ) : ext::UnaryNode < ext::smart_ptr < FormalRTEElement < SymbolType, RankType > >, ext::smart_ptr < const FormalRTEElement < SymbolType, RankType > >, FormalRTEIteration < SymbolType, RankType > > ( ext::smart_ptr < FormalRTEElement < SymbolType, RankType > > ( std::move ( element ).plunder ( ) ) ), m_substitutionSymbol ( ext::smart_ptr < FormalRTESymbolSubst < SymbolType, RankType > > ( std::move ( substitutionSymbol ).plunder ( ) ) ) {
}

template < class SymbolType, class RankType >
FormalRTEIteration < SymbolType, RankType >::FormalRTEIteration ( const FormalRTEElement < SymbolType, RankType > & element, FormalRTESymbolSubst < SymbolType, RankType > substitutionSymbol ) : FormalRTEIteration ( ext::move_copy ( element ), std::move ( substitutionSymbol ) ) {
}

template < class SymbolType, class RankType >
const FormalRTEElement < SymbolType, RankType > & FormalRTEIteration < SymbolType, RankType >::getElement ( ) const {
	return * this->getChild ( );
}

template < class SymbolType, class RankType >
FormalRTEElement < SymbolType, RankType > & FormalRTEIteration < SymbolType, RankType >::getElement ( ) {
	return * this->getChild ( );
}

template < class SymbolType, class RankType >
const FormalRTESymbolSubst < SymbolType, RankType > & FormalRTEIteration < SymbolType, RankType >::getSubstitutionSymbol ( ) const {
	return * m_substitutionSymbol;
}

template < class SymbolType, class RankType >
FormalRTESymbolSubst < SymbolType, RankType > & FormalRTEIteration < SymbolType, RankType >::getSubstitutionSymbol ( ) {
	return * m_substitutionSymbol;
}

template < class SymbolType, class RankType >
void FormalRTEIteration < SymbolType, RankType >::setElement ( const FormalRTEElement < SymbolType, RankType > & element ) {
	setElement ( ext::move_copy ( element ) );
}

template < class SymbolType, class RankType >
void FormalRTEIteration < SymbolType, RankType >::setElement ( FormalRTEElement < SymbolType, RankType > && elementParam ) {
	setChild ( ext::smart_ptr < FormalRTEElement < SymbolType, RankType > > ( std::move ( elementParam ).plunder ( ) ) );
}

template < class SymbolType, class RankType >
void FormalRTEIteration < SymbolType, RankType >::setSubstitutionSymbol ( FormalRTESymbolSubst < SymbolType, RankType > symbol ) {
	m_substitutionSymbol = ext::smart_ptr < FormalRTESymbolSubst < SymbolType, RankType > > ( std::move ( symbol ).plunder ( ) );
}

template < class SymbolType, class RankType >
FormalRTEIteration < SymbolType, RankType > * FormalRTEIteration < SymbolType, RankType >::clone ( ) const {
	return new FormalRTEIteration ( * this );
}

template < class SymbolType, class RankType >
FormalRTEIteration < SymbolType, RankType > * FormalRTEIteration < SymbolType, RankType >::plunder ( ) && {
	return new FormalRTEIteration ( std::move ( * this ) );
}

template < class SymbolType, class RankType >
int FormalRTEIteration < SymbolType, RankType >::compare ( const FormalRTEIteration & other ) const {
	auto first = ext::tie ( getElement ( ), getSubstitutionSymbol ( ) );
	auto second = ext::tie ( other.getElement ( ), other.getSubstitutionSymbol ( ) );

	static ext::compare < decltype ( first ) > comp;

	return comp ( first, second );
}

template < class SymbolType, class RankType >
void FormalRTEIteration < SymbolType, RankType >::operator >>( std::ostream & out ) const {
	out << "(RTEFormalRTEIteration " << * m_substitutionSymbol << " " << getElement ( ) << ")";
}

template < class SymbolType, class RankType >
bool FormalRTEIteration < SymbolType, RankType >::testSymbol ( const common::ranked_symbol < SymbolType, RankType > & symbol ) const {
	return this->getChild ( )->testSymbol ( symbol );
}

template < class SymbolType, class RankType >
void FormalRTEIteration < SymbolType, RankType >::computeMinimalAlphabet ( ext::set < common::ranked_symbol < SymbolType, RankType > > & alphabetF, ext::set < common::ranked_symbol < SymbolType, RankType > > & alphabetK ) const {
	alphabetK.insert ( m_substitutionSymbol->getSymbol ( ) );
	this->getChild ( )->computeMinimalAlphabet ( alphabetF, alphabetK );
}

template < class SymbolType, class RankType >
bool FormalRTEIteration < SymbolType, RankType >::checkAlphabet ( const ext::set < common::ranked_symbol < SymbolType, RankType > > & alphabetF, const ext::set < common::ranked_symbol < SymbolType, RankType > > & alphabetK ) const {
	return alphabetK.count ( getSubstitutionSymbol ( ).getSymbol ( ) ) > 0 && getElement ( ).checkAlphabet ( alphabetF, alphabetK );
}

template < class SymbolType, class RankType >
FormalRTEIteration < SymbolType, RankType >::operator std::string ( ) const {
	std::stringstream ss;
	ss << * this;
	return ss.str ( );
}

} /* namespace rte */
#endif /* FORMAL_RTE_ITERATION_H_ */
